package dev.pfaff.glass.opengl;

import dev.pfaff.glass.opengl.enums.GLStringName;

import static java.nio.charset.StandardCharsets.US_ASCII;

/**
 * @param major
 * @param minor
 * @param raw   the version as reported by {@link GL#glGetString}{@code
 * (}{@link dev.pfaff.glass.opengl.enums.GLStringName#VERSION VERSION}{@code )}
 */
public record GLVersion(int major, int minor, String raw, API api) {
	public static final String ES_PREFIX = "OpenGL ES ";

	public static GLVersion get(GL gl) throws ParseException {
		var raw = gl.glGetString(GLStringName.VERSION).ms().reinterpret(Long.MAX_VALUE).getString(0L, US_ASCII);
		try {
			var isES = raw.startsWith(ES_PREFIX);
			var start = isES ? ES_PREFIX.length() : 0;
			var i = raw.indexOf('.', start);
			var j = i+1;
			for (; j < raw.length(); j++) {
				var c = raw.charAt(j);
				if (c < '0' || c > '9') break;
			}
			int major = Integer.parseInt(raw.substring(start, i));
			int minor = Integer.parseInt(raw.substring(i+1, j));
			return new GLVersion(major, minor, raw, isES ? API.OpenGLES : API.OpenGL);
		} catch (Throwable t) {
			throw new ParseException(raw, t);
		}
	}

	public static final class ParseException extends RuntimeException {
		private final String version;

		private ParseException(String version, Throwable cause) {
			super("Failed to parse OpenGL version '" + version + "'", cause);
			this.version = version;
		}

		public String version() {
			return version;
		}
	}

	public enum API {
		OpenGL,
		OpenGLES,
	}
}
