package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLBlendingFactor implements GLEnum {
	ZERO(0x0),
	ONE(0x1),
	SRC_COLOR(0x300),
	ONE_MINUS_SRC_COLOR(0x301),
	SRC_ALPHA(0x302),
	ONE_MINUS_SRC_ALPHA(0x303),
	DST_ALPHA(0x304),
	ONE_MINUS_DST_ALPHA(0x305),
	DST_COLOR(0x306),
	ONE_MINUS_DST_COLOR(0x307),
	SRC_ALPHA_SATURATE(0x308),
	CONSTANT_COLOR(0x8001),
	ONE_MINUS_CONSTANT_COLOR(0x8002),
	CONSTANT_ALPHA(0x8003),
	ONE_MINUS_CONSTANT_ALPHA(0x8004),
	SRC1_ALPHA(0x8589),
	SRC1_COLOR(0x88f9),
	ONE_MINUS_SRC1_COLOR(0x88fa),
	ONE_MINUS_SRC1_ALPHA(0x88fb);
	
	private final int glEnum;

	GLBlendingFactor(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLBlendingFactor> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLBlendingFactor fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x0 -> ZERO;
			case 0x1 -> ONE;
			case 0x300 -> SRC_COLOR;
			case 0x301 -> ONE_MINUS_SRC_COLOR;
			case 0x302 -> SRC_ALPHA;
			case 0x303 -> ONE_MINUS_SRC_ALPHA;
			case 0x304 -> DST_ALPHA;
			case 0x305 -> ONE_MINUS_DST_ALPHA;
			case 0x306 -> DST_COLOR;
			case 0x307 -> ONE_MINUS_DST_COLOR;
			case 0x308 -> SRC_ALPHA_SATURATE;
			case 0x8001 -> CONSTANT_COLOR;
			case 0x8002 -> ONE_MINUS_CONSTANT_COLOR;
			case 0x8003 -> CONSTANT_ALPHA;
			case 0x8004 -> ONE_MINUS_CONSTANT_ALPHA;
			case 0x8589 -> SRC1_ALPHA;
			case 0x88f9 -> SRC1_COLOR;
			case 0x88fa -> ONE_MINUS_SRC1_COLOR;
			case 0x88fb -> ONE_MINUS_SRC1_ALPHA;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLBlendingFactor");
		};
	}
}