package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLInterleavedArrayFormat implements GLEnum {
	V2F(0x2a20),
	V3F(0x2a21),
	C4UB_V2F(0x2a22),
	C4UB_V3F(0x2a23),
	C3F_V3F(0x2a24),
	N3F_V3F(0x2a25),
	C4F_N3F_V3F(0x2a26),
	T2F_V3F(0x2a27),
	T4F_V4F(0x2a28),
	T2F_C4UB_V3F(0x2a29),
	T2F_C3F_V3F(0x2a2a),
	T2F_N3F_V3F(0x2a2b),
	T2F_C4F_N3F_V3F(0x2a2c),
	T4F_C4F_N3F_V4F(0x2a2d);
	
	private final int glEnum;

	GLInterleavedArrayFormat(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLInterleavedArrayFormat> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLInterleavedArrayFormat fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x2a20 -> V2F;
			case 0x2a21 -> V3F;
			case 0x2a22 -> C4UB_V2F;
			case 0x2a23 -> C4UB_V3F;
			case 0x2a24 -> C3F_V3F;
			case 0x2a25 -> N3F_V3F;
			case 0x2a26 -> C4F_N3F_V3F;
			case 0x2a27 -> T2F_V3F;
			case 0x2a28 -> T4F_V4F;
			case 0x2a29 -> T2F_C4UB_V3F;
			case 0x2a2a -> T2F_C3F_V3F;
			case 0x2a2b -> T2F_N3F_V3F;
			case 0x2a2c -> T2F_C4F_N3F_V3F;
			case 0x2a2d -> T4F_C4F_N3F_V4F;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLInterleavedArrayFormat");
		};
	}
}