package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLMapTarget implements GLEnum {
	MAP1_COLOR_4(0xd90),
	MAP1_INDEX(0xd91),
	MAP1_NORMAL(0xd92),
	MAP1_TEXTURE_COORD_1(0xd93),
	MAP1_TEXTURE_COORD_2(0xd94),
	MAP1_TEXTURE_COORD_3(0xd95),
	MAP1_TEXTURE_COORD_4(0xd96),
	MAP1_VERTEX_3(0xd97),
	MAP1_VERTEX_4(0xd98),
	MAP2_COLOR_4(0xdb0),
	MAP2_INDEX(0xdb1),
	MAP2_NORMAL(0xdb2),
	MAP2_TEXTURE_COORD_1(0xdb3),
	MAP2_TEXTURE_COORD_2(0xdb4),
	MAP2_TEXTURE_COORD_3(0xdb5),
	MAP2_TEXTURE_COORD_4(0xdb6),
	MAP2_VERTEX_3(0xdb7),
	MAP2_VERTEX_4(0xdb8),
	GEOMETRY_DEFORMATION_SGIX(0x8194),
	TEXTURE_DEFORMATION_SGIX(0x8195);
	
	private final int glEnum;

	GLMapTarget(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLMapTarget> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLMapTarget fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xd90 -> MAP1_COLOR_4;
			case 0xd91 -> MAP1_INDEX;
			case 0xd92 -> MAP1_NORMAL;
			case 0xd93 -> MAP1_TEXTURE_COORD_1;
			case 0xd94 -> MAP1_TEXTURE_COORD_2;
			case 0xd95 -> MAP1_TEXTURE_COORD_3;
			case 0xd96 -> MAP1_TEXTURE_COORD_4;
			case 0xd97 -> MAP1_VERTEX_3;
			case 0xd98 -> MAP1_VERTEX_4;
			case 0xdb0 -> MAP2_COLOR_4;
			case 0xdb1 -> MAP2_INDEX;
			case 0xdb2 -> MAP2_NORMAL;
			case 0xdb3 -> MAP2_TEXTURE_COORD_1;
			case 0xdb4 -> MAP2_TEXTURE_COORD_2;
			case 0xdb5 -> MAP2_TEXTURE_COORD_3;
			case 0xdb6 -> MAP2_TEXTURE_COORD_4;
			case 0xdb7 -> MAP2_VERTEX_3;
			case 0xdb8 -> MAP2_VERTEX_4;
			case 0x8194 -> GEOMETRY_DEFORMATION_SGIX;
			case 0x8195 -> TEXTURE_DEFORMATION_SGIX;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLMapTarget");
		};
	}
}