package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLMemoryBarrierMask implements GLEnum {
	VERTEX_ATTRIB_ARRAY_BARRIER_BIT(0x1),
	ELEMENT_ARRAY_BARRIER_BIT(0x2),
	UNIFORM_BARRIER_BIT(0x4),
	TEXTURE_FETCH_BARRIER_BIT(0x8),
	SHADER_GLOBAL_ACCESS_BARRIER_BIT_NV(0x10),
	SHADER_IMAGE_ACCESS_BARRIER_BIT(0x20),
	COMMAND_BARRIER_BIT(0x40),
	PIXEL_BUFFER_BARRIER_BIT(0x80),
	TEXTURE_UPDATE_BARRIER_BIT(0x100),
	BUFFER_UPDATE_BARRIER_BIT(0x200),
	FRAMEBUFFER_BARRIER_BIT(0x400),
	TRANSFORM_FEEDBACK_BARRIER_BIT(0x800),
	ATOMIC_COUNTER_BARRIER_BIT(0x1000),
	SHADER_STORAGE_BARRIER_BIT(0x2000),
	CLIENT_MAPPED_BUFFER_BARRIER_BIT(0x4000),
	QUERY_BUFFER_BARRIER_BIT(0x8000),
	ALL_BARRIER_BITS(0xffffffff);
	
	private final int glEnum;

	GLMemoryBarrierMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLMemoryBarrierMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLMemoryBarrierMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> VERTEX_ATTRIB_ARRAY_BARRIER_BIT;
			case 0x2 -> ELEMENT_ARRAY_BARRIER_BIT;
			case 0x4 -> UNIFORM_BARRIER_BIT;
			case 0x8 -> TEXTURE_FETCH_BARRIER_BIT;
			case 0x10 -> SHADER_GLOBAL_ACCESS_BARRIER_BIT_NV;
			case 0x20 -> SHADER_IMAGE_ACCESS_BARRIER_BIT;
			case 0x40 -> COMMAND_BARRIER_BIT;
			case 0x80 -> PIXEL_BUFFER_BARRIER_BIT;
			case 0x100 -> TEXTURE_UPDATE_BARRIER_BIT;
			case 0x200 -> BUFFER_UPDATE_BARRIER_BIT;
			case 0x400 -> FRAMEBUFFER_BARRIER_BIT;
			case 0x800 -> TRANSFORM_FEEDBACK_BARRIER_BIT;
			case 0x1000 -> ATOMIC_COUNTER_BARRIER_BIT;
			case 0x2000 -> SHADER_STORAGE_BARRIER_BIT;
			case 0x4000 -> CLIENT_MAPPED_BUFFER_BARRIER_BIT;
			case 0x8000 -> QUERY_BUFFER_BARRIER_BIT;
			case 0xffffffff -> ALL_BARRIER_BITS;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLMemoryBarrierMask");
		};
	}
}