package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPixelFormat implements GLEnum {
	UNSIGNED_SHORT(0x1403),
	UNSIGNED_INT(0x1405),
	COLOR_INDEX(0x1900),
	STENCIL_INDEX(0x1901),
	DEPTH_COMPONENT(0x1902),
	RED(0x1903),
	GREEN(0x1904),
	BLUE(0x1905),
	ALPHA(0x1906),
	RGB(0x1907),
	RGBA(0x1908),
	LUMINANCE(0x1909),
	LUMINANCE_ALPHA(0x190a),
	ABGR_EXT(0x8000),
	CMYK_EXT(0x800c),
	CMYKA_EXT(0x800d),
	BGR(0x80e0),
	BGRA(0x80e1),
	YCRCB_422_SGIX(0x81bb),
	YCRCB_444_SGIX(0x81bc),
	RG(0x8227),
	RG_INTEGER(0x8228),
	DEPTH_STENCIL(0x84f9),
	RED_INTEGER(0x8d94),
	GREEN_INTEGER(0x8d95),
	BLUE_INTEGER(0x8d96),
	RGB_INTEGER(0x8d98),
	RGBA_INTEGER(0x8d99),
	BGR_INTEGER(0x8d9a),
	BGRA_INTEGER(0x8d9b);
	
	private final int glEnum;

	GLPixelFormat(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPixelFormat> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPixelFormat fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1403 -> UNSIGNED_SHORT;
			case 0x1405 -> UNSIGNED_INT;
			case 0x1900 -> COLOR_INDEX;
			case 0x1901 -> STENCIL_INDEX;
			case 0x1902 -> DEPTH_COMPONENT;
			case 0x1903 -> RED;
			case 0x1904 -> GREEN;
			case 0x1905 -> BLUE;
			case 0x1906 -> ALPHA;
			case 0x1907 -> RGB;
			case 0x1908 -> RGBA;
			case 0x1909 -> LUMINANCE;
			case 0x190a -> LUMINANCE_ALPHA;
			case 0x8000 -> ABGR_EXT;
			case 0x800c -> CMYK_EXT;
			case 0x800d -> CMYKA_EXT;
			case 0x80e0 -> BGR;
			case 0x80e1 -> BGRA;
			case 0x81bb -> YCRCB_422_SGIX;
			case 0x81bc -> YCRCB_444_SGIX;
			case 0x8227 -> RG;
			case 0x8228 -> RG_INTEGER;
			case 0x84f9 -> DEPTH_STENCIL;
			case 0x8d94 -> RED_INTEGER;
			case 0x8d95 -> GREEN_INTEGER;
			case 0x8d96 -> BLUE_INTEGER;
			case 0x8d98 -> RGB_INTEGER;
			case 0x8d99 -> RGBA_INTEGER;
			case 0x8d9a -> BGR_INTEGER;
			case 0x8d9b -> BGRA_INTEGER;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPixelFormat");
		};
	}
}