package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPrimitiveType implements GLEnum {
	POINTS(0x0),
	LINES(0x1),
	LINE_LOOP(0x2),
	LINE_STRIP(0x3),
	TRIANGLES(0x4),
	TRIANGLE_STRIP(0x5),
	TRIANGLE_FAN(0x6),
	QUADS(0x7),
	QUAD_STRIP(0x8),
	POLYGON(0x9),
	LINES_ADJACENCY(0xa),
	LINE_STRIP_ADJACENCY(0xb),
	TRIANGLES_ADJACENCY(0xc),
	TRIANGLE_STRIP_ADJACENCY(0xd),
	PATCHES(0xe);
	
	private final int glEnum;

	GLPrimitiveType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPrimitiveType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPrimitiveType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x0 -> POINTS;
			case 0x1 -> LINES;
			case 0x2 -> LINE_LOOP;
			case 0x3 -> LINE_STRIP;
			case 0x4 -> TRIANGLES;
			case 0x5 -> TRIANGLE_STRIP;
			case 0x6 -> TRIANGLE_FAN;
			case 0x7 -> QUADS;
			case 0x8 -> QUAD_STRIP;
			case 0x9 -> POLYGON;
			case 0xa -> LINES_ADJACENCY;
			case 0xb -> LINE_STRIP_ADJACENCY;
			case 0xc -> TRIANGLES_ADJACENCY;
			case 0xd -> TRIANGLE_STRIP_ADJACENCY;
			case 0xe -> PATCHES;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPrimitiveType");
		};
	}
}