package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLSubgroupSupportedFeatures implements GLEnum {
	SUBGROUP_FEATURE_BASIC_BIT_KHR(0x1),
	SUBGROUP_FEATURE_VOTE_BIT_KHR(0x2),
	SUBGROUP_FEATURE_ARITHMETIC_BIT_KHR(0x4),
	SUBGROUP_FEATURE_BALLOT_BIT_KHR(0x8),
	SUBGROUP_FEATURE_SHUFFLE_BIT_KHR(0x10),
	SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT_KHR(0x20),
	SUBGROUP_FEATURE_CLUSTERED_BIT_KHR(0x40),
	SUBGROUP_FEATURE_QUAD_BIT_KHR(0x80),
	SUBGROUP_FEATURE_PARTITIONED_BIT_NV(0x100);
	
	private final int glEnum;

	GLSubgroupSupportedFeatures(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLSubgroupSupportedFeatures> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLSubgroupSupportedFeatures fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> SUBGROUP_FEATURE_BASIC_BIT_KHR;
			case 0x2 -> SUBGROUP_FEATURE_VOTE_BIT_KHR;
			case 0x4 -> SUBGROUP_FEATURE_ARITHMETIC_BIT_KHR;
			case 0x8 -> SUBGROUP_FEATURE_BALLOT_BIT_KHR;
			case 0x10 -> SUBGROUP_FEATURE_SHUFFLE_BIT_KHR;
			case 0x20 -> SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT_KHR;
			case 0x40 -> SUBGROUP_FEATURE_CLUSTERED_BIT_KHR;
			case 0x80 -> SUBGROUP_FEATURE_QUAD_BIT_KHR;
			case 0x100 -> SUBGROUP_FEATURE_PARTITIONED_BIT_NV;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLSubgroupSupportedFeatures");
		};
	}
}