package dev.pfaff.glass.posix;

import dev.pfaff.glass.Host;
import dev.pfaff.glass.OS;

import java.lang.foreign.*;
import java.lang.invoke.MethodHandle;

import static dev.pfaff.glass.Interop.ADDRESS_INT;
import static java.lang.foreign.ValueLayout.*;

public final class PThread {
	private static final MethodHandle MH_pthread_threadid_np;
	private static final MethodHandle MH_pthread_self;
	private static final MethodHandle MH_pthread_main_thread_np;
	private static final MethodHandle MH_pthread_main_np;

	static {
		var lookup = switch (OS.HOST) {
			case OS.macOS _ -> SymbolLookup.libraryLookup("/usr/lib/system/libsystem_pthread.dylib", Arena.ofAuto());
			default -> throw Host.makeUnsupportedOSArchException();
		};
		var linker = Linker.nativeLinker();
		MH_pthread_threadid_np = linker.downcallHandle(lookup.findOrThrow("pthread_threadid_np"),
				FunctionDescriptor.of(JAVA_INT, ADDRESS, ADDRESS.withTargetLayout(ADDRESS_INT)),
				Linker.Option.critical(true));
		MH_pthread_self = linker.downcallHandle(lookup.findOrThrow("pthread_self"),
				FunctionDescriptor.of(ADDRESS),
				Linker.Option.critical(false));
		MH_pthread_main_thread_np = linker.downcallHandle(lookup.findOrThrow("pthread_main_thread_np"),
				FunctionDescriptor.of(ADDRESS),
				Linker.Option.critical(false));
		MH_pthread_main_np = linker.downcallHandle(lookup.findOrThrow("pthread_main_np"),
				FunctionDescriptor.of(JAVA_BOOLEAN),
				Linker.Option.critical(false));
	}

	public static MemorySegment pthread_self() {
		try {
			return (MemorySegment) MH_pthread_self.invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @param pthread_t may be null, in which case it is treated as though it were the result of calling
	 *                  {@link #pthread_self()}.
	 */
	public static int pthread_threadid_np(MemorySegment pthread_t) {
		var slot = new long[1];
		try {
			// NULL == self
			return (int) MH_pthread_threadid_np.invokeExact(pthread_t, MemorySegment.ofArray(slot));
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public static MemorySegment pthread_main_thread_np() {
		try {
			return (MemorySegment) MH_pthread_main_thread_np.invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public static boolean pthread_main_np() {
		try {
			return (boolean) MH_pthread_main_np.invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
