package dev.pfaff.glass.posix;

import dev.pfaff.glass.Host;
import dev.pfaff.glass.OS;

import java.lang.foreign.FunctionDescriptor;
import java.lang.foreign.Linker;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.ValueLayout;
import java.lang.invoke.MethodHandle;
import java.util.Set;

import static dev.pfaff.glass.Interop.ADDRESS_INT;
import static dev.pfaff.glass.apple.SystemFramework.SYSTEM_FRAMEWORK;
import static java.lang.foreign.ValueLayout.ADDRESS;
import static java.lang.foreign.ValueLayout.JAVA_INT;

/**
 * Bindings to C standard library on POSIX systems.
 */
public final class Posix {
	public static final MethodHandle MH_mprotect;

	public static final ValueLayout ML_MemProt = JAVA_INT.withName(MemProt.class.getSimpleName());

	static {
		var libc = switch (OS.HOST) {
			case OS.macOS _ -> SYSTEM_FRAMEWORK;
			default -> throw Host.makeUnsupportedOSArchException();
		};

		var linker = Linker.nativeLinker();
		MH_mprotect = linker.downcallHandle(libc.find("mprotect").get(),
				FunctionDescriptor.of(JAVA_INT, ADDRESS, ADDRESS_INT, ML_MemProt));
	}

	public static void mprotect(MemorySegment memory, Set<MemProt> protection) {
		var prot = protection.stream().mapToInt(MemProt::mask).reduce(0, (a, b) -> a | b);
		int result;
		try {
			result = (int) MH_mprotect.invokeExact(memory, memory.byteSize(), prot);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
		if (result != 0) throw new RuntimeException();
	}
}
